use actix_session::{storage::RedisSessionStore, SessionMiddleware};
use actix_web::{cookie::Key, middleware::Logger, web, App, HttpResponse, HttpServer};
use db::Database;
use std::env;

mod controller;
mod db;

#[actix_web::main]
async fn main() -> std::io::Result<()> {
    env_logger::init_from_env(env_logger::Env::new().default_filter_or("info"));
    dotenvy::dotenv().ok();

    let database_url = build_database_url();

    let database = Database::new(database_url.into()).await.unwrap();

    let redis_conn = connect_to_redis_database().await;

    HttpServer::new(move || {
        App::new()
            .app_data(web::Data::new(database.clone()))
            .wrap(Logger::default())
            .wrap(SessionMiddleware::new(
                redis_conn.clone(),
                // use dotenvy here to get SECRET_KEY (if we want to have the same on every startup)
                Key::generate(),
            ))
            .configure(controller::register_controllers)
    })
    .bind(("0.0.0.0", 8080))?
    .run()
    .await
}

async fn connect_to_redis_database() -> RedisSessionStore {
    // Build the redis Connection from the .env file Variables
    let redis_connection_string = "redis://127.0.0.1:6379";
    let store = RedisSessionStore::new(redis_connection_string)
        .await
        .unwrap();

    return store;
}


fn build_database_url() -> String {
    let db_user = dotenvy::var("DB_USER").unwrap_or("pgg".to_owned());
    let db_name = dotenvy::var("DB_NAME").unwrap_or("pgg".to_owned());
    let db_password = dotenvy::var("DB_PASSWORD").unwrap_or("pgg".to_owned());
    let db_host = dotenvy::var("DB_HOST").expect("DB_HOST must be set");
    let db_port = dotenvy::var("DB_PORT").map(|x| x.parse::<u16>().expect("DB_PORT is not a valid port")).unwrap_or(5432);

    format!("postgresql://{}:{}@{}:{}/{}", db_user, db_password, db_host, db_port, db_name)
}